@@ -1,13 +1,13 @@
 """Routines for finding the sources that mypy will check"""
 
+from __future__ import annotations
+
 import functools
 import os
+from typing import Final, Sequence
 
-from typing import List, Sequence, Set, Tuple, Optional
-from typing_extensions import Final
-
-from mypy.modulefinder import BuildSource, PYTHON_EXTENSIONS, mypy_path, matches_exclude
 from mypy.fscache import FileSystemCache
+from mypy.modulefinder import PYTHON_EXTENSIONS, BuildSource, matches_exclude, mypy_path
 from mypy.options import Options
 
 PY_EXTENSIONS: Final = tuple(PYTHON_EXTENSIONS)
@@ -17,9 +17,12 @@ class InvalidSourceList(Exception):
     """Exception indicating a problem in the list of sources given to mypy."""
 
 
-def create_source_list(paths: Sequence[str], options: Options,
-                       fscache: Optional[FileSystemCache] = None,
-                       allow_empty_dir: bool = False) -> List[BuildSource]:
+def create_source_list(
+    paths: Sequence[str],
+    options: Options,
+    fscache: FileSystemCache | None = None,
+    allow_empty_dir: bool = False,
+) -> list[BuildSource]:
     """From a list of source files/directories, makes a list of BuildSources.
 
     Raises InvalidSourceList on errors.
@@ -37,9 +40,7 @@ def create_source_list(paths: Sequence[str], options: Options,
         elif fscache.isdir(path):
             sub_sources = finder.find_sources_in_dir(path)
             if not sub_sources and not allow_empty_dir:
-                raise InvalidSourceList(
-                    "There are no .py[i] files in directory '{}'".format(path)
-                )
+                raise InvalidSourceList(f"There are no .py[i] files in directory '{path}'")
             sources.extend(sub_sources)
         else:
             mod = os.path.basename(path) if options.scripts_are_modules else None
@@ -47,7 +48,7 @@ def create_source_list(paths: Sequence[str], options: Options,
     return sources
 
 
-def keyfunc(name: str) -> Tuple[bool, int, str]:
+def keyfunc(name: str) -> tuple[bool, int, str]:
     """Determines sort order for directory listing.
 
     The desirable properties are:
@@ -70,7 +71,7 @@ def normalise_package_base(root: str) -> str:
     return root
 
 
-def get_explicit_package_bases(options: Options) -> Optional[List[str]]:
+def get_explicit_package_bases(options: Options) -> list[str] | None:
     """Returns explicit package bases to use if the option is enabled, or None if disabled.
 
     We currently use MYPYPATH and the current directory as the package bases. In the future,
@@ -98,10 +99,10 @@ class SourceFinder:
         assert self.explicit_package_bases
         return normalise_package_base(path) in self.explicit_package_bases
 
-    def find_sources_in_dir(self, path: str) -> List[BuildSource]:
+    def find_sources_in_dir(self, path: str) -> list[BuildSource]:
         sources = []
 
-        seen: Set[str] = set()
+        seen: set[str] = set()
         names = sorted(self.fscache.listdir(path), key=keyfunc)
         for name in names:
             # Skip certain names altogether
@@ -109,9 +110,7 @@ class SourceFinder:
                 continue
             subpath = os.path.join(path, name)
 
-            if matches_exclude(
-                subpath, self.exclude, self.fscache, self.verbosity >= 2
-            ):
+            if matches_exclude(subpath, self.exclude, self.fscache, self.verbosity >= 2):
                 continue
 
             if self.fscache.isdir(subpath):
@@ -128,7 +127,7 @@ class SourceFinder:
 
         return sources
 
-    def crawl_up(self, path: str) -> Tuple[str, str]:
+    def crawl_up(self, path: str) -> tuple[str, str]:
         """Given a .py[i] filename, return module and base directory.
 
         For example, given "xxx/yyy/foo/bar.py", we might return something like:
@@ -157,11 +156,11 @@ class SourceFinder:
         module = module_join(parent_module, module_name)
         return module, base_dir
 
-    def crawl_up_dir(self, dir: str) -> Tuple[str, str]:
+    def crawl_up_dir(self, dir: str) -> tuple[str, str]:
         return self._crawl_up_helper(dir) or ("", dir)
 
-    @functools.lru_cache()  # noqa: B019
-    def _crawl_up_helper(self, dir: str) -> Optional[Tuple[str, str]]:
+    @functools.lru_cache  # noqa: B019
+    def _crawl_up_helper(self, dir: str) -> tuple[str, str] | None:
         """Given a directory, maybe returns module and base directory.
 
         We return a non-None value if we were able to find something clearly intended as a base
@@ -176,7 +175,7 @@ class SourceFinder:
             return "", dir
 
         parent, name = os.path.split(dir)
-        if name.endswith('-stubs'):
+        if name.endswith("-stubs"):
             name = name[:-6]  # PEP-561 stub-only directory
 
         # recurse if there's an __init__.py
@@ -185,7 +184,7 @@ class SourceFinder:
             if not name.isidentifier():
                 # in most cases the directory name is invalid, we'll just stop crawling upwards
                 # but if there's an __init__.py in the directory, something is messed up
-                raise InvalidSourceList("{} is not a valid Python package name".format(name))
+                raise InvalidSourceList(f"{name} is not a valid Python package name")
             # we're definitely a package, so we always return a non-None value
             mod_prefix, base_dir = self.crawl_up_dir(parent)
             return module_join(mod_prefix, name), base_dir
@@ -210,7 +209,7 @@ class SourceFinder:
         mod_prefix, base_dir = result
         return module_join(mod_prefix, name), base_dir
 
-    def get_init_file(self, dir: str) -> Optional[str]:
+    def get_init_file(self, dir: str) -> str | None:
         """Check whether a directory contains a file named __init__.py[i].
 
         If so, return the file's name (with dir prefixed).  If not, return None.
@@ -218,10 +217,10 @@ class SourceFinder:
         This prefers .pyi over .py (because of the ordering of PY_EXTENSIONS).
         """
         for ext in PY_EXTENSIONS:
-            f = os.path.join(dir, '__init__' + ext)
+            f = os.path.join(dir, "__init__" + ext)
             if self.fscache.isfile(f):
                 return f
-            if ext == '.py' and self.fscache.init_under_package_root(f):
+            if ext == ".py" and self.fscache.init_under_package_root(f):
                 return f
         return None
 
@@ -229,16 +228,16 @@ class SourceFinder:
 def module_join(parent: str, child: str) -> str:
     """Join module ids, accounting for a possibly empty parent."""
     if parent:
-        return parent + '.' + child
+        return parent + "." + child
     return child
 
 
-def strip_py(arg: str) -> Optional[str]:
+def strip_py(arg: str) -> str | None:
     """Strip a trailing .py or .pyi suffix.
 
     Return None if no such suffix is found.
     """
     for ext in PY_EXTENSIONS:
         if arg.endswith(ext):
-            return arg[:-len(ext)]
+            return arg[: -len(ext)]
     return None
